@@ -1,15 +1,12 @@
-#ifndef _XTABLES_COMPAT_H
-#define _XTABLES_COMPAT_H 1
-
+#pragma once
 #include <linux/kernel.h>
 #include <linux/version.h>
 #include "compat_skbuff.h"
-#include "compat_xtnu.h"
 
 #define DEBUGP Use__pr_debug__instead
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 15, 0)
-#	warning Kernels below 4.15 not supported.
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 16, 0)
+#	warning Kernels below 4.16 not supported.
 #endif
 
 #if defined(CONFIG_NF_CONNTRACK) || defined(CONFIG_NF_CONNTRACK_MODULE)
@@ -48,4 +45,4 @@ static inline struct net *par_net(const struct xt_action_param *par)
 #	define proc_release release
 #endif
 
-#endif /* _XTABLES_COMPAT_H */
+extern void *HX_memmem(const void *, size_t, const void *, size_t);
