@@ -19,6 +19,35 @@ MODULE_AUTHOR("Eicke Friedrich/Klaus Degner <ipp2p@ipp2p.org>");
 MODULE_DESCRIPTION("An extension to iptables to identify P2P traffic.");
 MODULE_LICENSE("GPL");
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 1, 0)
+static inline unsigned int
+ip_transport_len(const struct sk_buff *skb)
+{
+        return ntohs(ip_hdr(skb)->tot_len) - skb_network_header_len(skb);
+}
+static inline unsigned int
+ipv6_transport_len(const struct sk_buff *skb)
+{
+        return ntohs(ipv6_hdr(skb)->payload_len) + sizeof(struct ipv6hdr) -
+               skb_network_header_len(skb);
+}
+#endif
+
+struct ipp2p_result_printer {
+	const union nf_inet_addr *saddr, *daddr;
+	short sport, dport;
+	void (*print)(const union nf_inet_addr *, short, const union nf_inet_addr *, short, bool, unsigned int);
+};
+
+static void
+print_result(const struct ipp2p_result_printer *rp, bool result,
+             unsigned int hlen)
+{
+	rp->print(rp->saddr, rp->sport,
+		  rp->daddr, rp->dport,
+		  result, hlen);
+}
+
 /* Search for UDP eDonkey/eMule/Kad commands */
 static unsigned int
 udp_search_edk(const unsigned char *t, const unsigned int packet_len)
@@ -807,17 +836,136 @@ static const struct {
 	{0},
 };
 
+static void
+ipp2p_print_result_tcp4(const union nf_inet_addr *saddr, short sport,
+                        const union nf_inet_addr *daddr, short dport,
+                        bool p2p_result, unsigned int hlen)
+{
+	printk("IPP2P.debug:TCP-match: %d from: %pI4:%hu to: %pI4:%hu Length: %u\n",
+	       p2p_result, &saddr->ip, sport, &daddr->ip, dport, hlen);
+}
+
+static void
+ipp2p_print_result_tcp6(const union nf_inet_addr *saddr, short sport,
+                        const union nf_inet_addr *daddr, short dport,
+                        bool p2p_result, unsigned int hlen)
+{
+	printk("IPP2P.debug:TCP-match: %d from: %pI6:%hu to: %pI6:%hu Length: %u\n",
+	       p2p_result, &saddr->in6, sport, &daddr->in6, dport, hlen);
+}
+
+static bool
+ipp2p_mt_tcp(const struct ipt_p2p_info *info, const struct tcphdr *tcph,
+             const unsigned char *haystack, unsigned int hlen,
+             const struct ipp2p_result_printer *rp)
+{
+	size_t tcph_len = tcph->doff * 4;
+	bool p2p_result = false;
+	int i = 0;
+
+	if (tcph->fin) return 0;  /* if FIN bit is set bail out */
+	if (tcph->syn) return 0;  /* if SYN bit is set bail out */
+	if (tcph->rst) return 0;  /* if RST bit is set bail out */
+
+	if (hlen < tcph_len) {
+		if (info->debug)
+			pr_info("TCP header indicated packet larger than it is\n");
+		return 0;
+	}
+	if (hlen == tcph_len)
+		return 0;
+
+	haystack += tcph_len;
+	hlen     -= tcph_len;
+
+	while (matchlist[i].command) {
+		if ((info->cmd & matchlist[i].command) == matchlist[i].command &&
+		    hlen > matchlist[i].packet_len)
+		{
+			p2p_result = matchlist[i].function_name(haystack, hlen);
+			if (p2p_result)	{
+				if (info->debug)
+					print_result(rp, p2p_result, hlen);
+				return p2p_result;
+			}
+		}
+		i++;
+	}
+	return p2p_result;
+}
+
+static void
+ipp2p_print_result_udp4(const union nf_inet_addr *saddr, short sport,
+                        const union nf_inet_addr *daddr, short dport,
+                        bool p2p_result, unsigned int hlen)
+{
+	printk("IPP2P.debug:UDP-match: %d from: %pI4:%hu to: %pI4:%hu Length: %u\n",
+	       p2p_result, &saddr->ip, sport, &daddr->ip, dport, hlen);
+}
+
+static void
+ipp2p_print_result_udp6(const union nf_inet_addr *saddr, short sport,
+                        const union nf_inet_addr *daddr, short dport,
+                        bool p2p_result, unsigned int hlen)
+{
+	printk("IPP2P.debug:UDP-match: %d from: %pI6:%hu to: %pI6:%hu Length: %u\n",
+	       p2p_result, &saddr->in6, sport, &daddr->in6, dport, hlen);
+}
+
+static bool
+ipp2p_mt_udp(const struct ipt_p2p_info *info, const struct udphdr *udph,
+             const unsigned char *haystack, unsigned int hlen,
+             const struct ipp2p_result_printer *rp)
+{
+	size_t udph_len = sizeof(*udph);
+	bool p2p_result = false;
+	int i = 0;
+
+	if (hlen < udph_len) {
+		if (info->debug)
+			pr_info("UDP header indicated packet larger than it is\n");
+		return 0;
+	}
+	if (hlen == udph_len)
+		return 0;
+
+	haystack += udph_len;
+	hlen     -= udph_len;
+
+	while (udp_list[i].command) {
+		if ((info->cmd & udp_list[i].command) == udp_list[i].command &&
+		    hlen > udp_list[i].packet_len)
+		{
+			p2p_result = udp_list[i].function_name(haystack, hlen);
+			if (p2p_result) {
+				if (info->debug)
+					print_result(rp, p2p_result, hlen);
+				return p2p_result;
+			}
+		}
+		i++;
+	}
+	return p2p_result;
+}
+
 static bool
 ipp2p_mt(const struct sk_buff *skb, struct xt_action_param *par)
 {
 	const struct ipt_p2p_info *info = par->matchinfo;
-	const unsigned char  *haystack;
-	const struct iphdr *ip = ip_hdr(skb);
-	bool p2p_result = false;
-	int i = 0;
-	unsigned int hlen = ntohs(ip->tot_len) - ip_hdrlen(skb);	/* hlen = packet-data length */
+	struct ipp2p_result_printer printer;
+	union nf_inet_addr saddr, daddr;
+	const unsigned char *haystack;  /* packet data */
+	unsigned int hlen;              /* packet data length */
+	uint8_t family = xt_family(par);
+	int protocol;
 
-	/* must not be a fragment */
+	/*
+	 * must not be a fragment
+	 *
+	 * NB, `par->fragoff` may be zero for a fragmented IPv6 packet.
+	 * However, in that case the later call to `ipv6_find_hdr` will not find
+	 * a transport protocol, and so we will return 0 there.
+	 */
 	if (par->fragoff != 0) {
 		if (info->debug)
 			printk("IPP2P.match: offset found %d\n", par->fragoff);
@@ -831,103 +979,85 @@ ipp2p_mt(const struct sk_buff *skb, struct xt_action_param *par)
 		return 0;
 	}
 
-	haystack = skb_network_header(skb) + ip_hdrlen(skb);
+	if (family == NFPROTO_IPV4) {
+		const struct iphdr *ip = ip_hdr(skb);
+		saddr.ip = ip->saddr;
+		daddr.ip = ip->daddr;
+		protocol = ip->protocol;
+		hlen = ip_transport_len(skb);
+	} else {
+		const struct ipv6hdr *ip = ipv6_hdr(skb);
+		int thoff = 0;
 
-	switch (ip->protocol) {
+		saddr.in6 = ip->saddr;
+		daddr.in6 = ip->daddr;
+		protocol = ipv6_find_hdr(skb, &thoff, -1, NULL, NULL);
+		if (protocol < 0)
+			return 0;
+		hlen = ipv6_transport_len(skb);
+	}
+
+	printer.saddr = &saddr;
+	printer.daddr = &daddr;
+	haystack = skb_transport_header(skb);
+
+	switch (protocol) {
 	case IPPROTO_TCP:	/* what to do with a TCP packet */
 	{
-		const struct tcphdr *tcph = (const void *)ip + ip_hdrlen(skb);
+		const struct tcphdr *tcph = tcp_hdr(skb);
 
-		if (tcph->fin) return 0;  /* if FIN bit is set bail out */
-		if (tcph->syn) return 0;  /* if SYN bit is set bail out */
-		if (tcph->rst) return 0;  /* if RST bit is set bail out */
-
-		haystack += tcph->doff * 4; /* get TCP-Header-Size */
-		if (tcph->doff * 4 > hlen) {
-			if (info->debug)
-				pr_info("TCP header indicated packet larger than it is\n");
-			hlen = 0;
-		} else {
-			hlen -= tcph->doff * 4;
-		}
-		while (matchlist[i].command) {
-			if ((info->cmd & matchlist[i].command) == matchlist[i].command &&
-			    hlen > matchlist[i].packet_len)
-			{
-				p2p_result = matchlist[i].function_name(haystack, hlen);
-				if (p2p_result)	{
-					if (info->debug)
-						printk("IPP2P.debug:TCP-match: %d from: %pI4:%hu to: %pI4:%hu Length: %d\n",
-						       p2p_result, &ip->saddr,
-						       ntohs(tcph->source),
-						       &ip->daddr,
-						       ntohs(tcph->dest), hlen);
-					return p2p_result;
-				}
-			}
-			i++;
-		}
-		return p2p_result;
+		printer.sport = ntohs(tcph->source);
+		printer.dport = ntohs(tcph->dest);
+		printer.print = family == NFPROTO_IPV6 ?
+		                ipp2p_print_result_tcp6 : ipp2p_print_result_tcp4;
+		return ipp2p_mt_tcp(info, tcph, haystack, hlen, &printer);
 	}
-
-	case IPPROTO_UDP:	/* what to do with an UDP packet */
+	case IPPROTO_UDP:	/* what to do with a UDP packet */
 	case IPPROTO_UDPLITE:
 	{
-		const struct udphdr *udph = (const void *)ip + ip_hdrlen(skb);
+		const struct udphdr *udph = udp_hdr(skb);
 
-		haystack += sizeof(*udph);
-		if (sizeof(*udph) > hlen) {
-			if (info->debug)
-				pr_info("UDP header indicated packet larger than it is\n");
-			hlen = 0;
-		} else {
-			hlen -= sizeof(*udph);
-		}
-
-		while (udp_list[i].command) {
-			if ((info->cmd & udp_list[i].command) == udp_list[i].command &&
-			    hlen > udp_list[i].packet_len)
-			{
-				p2p_result = udp_list[i].function_name(haystack, hlen);
-				if (p2p_result) {
-					if (info->debug)
-						printk("IPP2P.debug:UDP-match: %d from: %pI4:%hu to: %pI4:%hu Length: %d\n",
-						       p2p_result, &ip->saddr,
-						       ntohs(udph->source),
-						       &ip->daddr,
-						       ntohs(udph->dest), hlen);
-					return p2p_result;
-				}
-			}
-			i++;
-		}
-		return p2p_result;
+		printer.sport = ntohs(udph->source);
+		printer.dport = ntohs(udph->dest);
+		printer.print = family == NFPROTO_IPV6 ?
+		                ipp2p_print_result_udp6 : ipp2p_print_result_udp4;
+		return ipp2p_mt_udp(info, udph, haystack, hlen, &printer);
 	}
-
 	default:
 		return 0;
 	}
 }
 
-static struct xt_match ipp2p_mt_reg __read_mostly = {
-	.name       = "ipp2p",
-	.revision   = 1,
-	.family     = NFPROTO_IPV4,
-	.match      = ipp2p_mt,
-	.matchsize  = sizeof(struct ipt_p2p_info),
-	.me         = THIS_MODULE,
+static struct xt_match ipp2p_mt_reg[] __read_mostly = {
+	{
+		.name       = "ipp2p",
+		.revision   = 1,
+		.family     = NFPROTO_IPV4,
+		.match      = ipp2p_mt,
+		.matchsize  = sizeof(struct ipt_p2p_info),
+		.me         = THIS_MODULE,
+	},
+	{
+		.name       = "ipp2p",
+		.revision   = 1,
+		.family     = NFPROTO_IPV6,
+		.match      = ipp2p_mt,
+		.matchsize  = sizeof(struct ipt_p2p_info),
+		.me         = THIS_MODULE,
+	},
 };
 
 static int __init ipp2p_mt_init(void)
 {
-	return xt_register_match(&ipp2p_mt_reg);
+	return xt_register_matches(ipp2p_mt_reg, ARRAY_SIZE(ipp2p_mt_reg));
 }
 
 static void __exit ipp2p_mt_exit(void)
 {
-	xt_unregister_match(&ipp2p_mt_reg);
+	xt_unregister_matches(ipp2p_mt_reg, ARRAY_SIZE(ipp2p_mt_reg));
 }
 
 module_init(ipp2p_mt_init);
 module_exit(ipp2p_mt_exit);
 MODULE_ALIAS("ipt_ipp2p");
+MODULE_ALIAS("ip6t_ipp2p");
